@@ -1,6 +1,9 @@
-from django.conf.urls import patterns, url
+from django.urls import re_path
 from django.contrib import admin
-from django.core.urlresolvers import reverse
+try:
+    from django.urls import reverse
+except ImportError:
+    from django.core.urlresolvers import reverse
 from django.http import HttpResponseRedirect
 from .models import FileBrowser
 from .settings import SHOW_IN_DASHBOARD
@@ -18,9 +21,9 @@ class FileBrowserAdmin(admin.ModelAdmin):
     def get_urls(self):
         opts = self.model._meta
         info = opts.app_label, (opts.model_name if hasattr(opts, 'model_name') else opts.module_name)
-        return patterns('',
-            url('^$', self.admin_site.admin_view(self.filebrowser_view), name='{0}_{1}_changelist'.format(*info)),
-        )
+        return [
+            re_path('^$', self.admin_site.admin_view(self.filebrowser_view), name='{0}_{1}_changelist'.format(*info)),
+        ]
 
     def filebrowser_view(self, request):
         return HttpResponseRedirect(reverse('filebrowser:fb_browse'))
