@@ -10,6 +10,7 @@ import time
 from django.core.files import File
 from django.utils.encoding import force_str
 from django.utils.functional import cached_property
+from six import python_2_unicode_compatible, string_types
 
 from filebrowser.settings import EXTENSIONS, VERSIONS, ADMIN_VERSIONS, VERSIONS_BASEDIR, VERSION_QUALITY, STRICT_PIL, IMAGE_MAXBLOCK, DEFAULT_PERMISSIONS
 from filebrowser.utils import path_strip, process_image
@@ -78,7 +79,9 @@ class FileListing():
         the sorted list of objects.
         """
         from operator import attrgetter
-        return sorted(seq, key=attrgetter(attr))
+        if isinstance(attr, string_types):  # Backward compatibility hack
+            attr = (attr, )
+        return sorted(seq, key=attrgetter(*attr))
 
     @cached_property
     def is_folder(self):
@@ -194,6 +197,7 @@ class FileListing():
         return len(self.files_walk_filtered())
 
 
+@python_2_unicode_compatible
 class FileObject():
     """
     The FileObject represents a file (or directory) on the server.
@@ -264,30 +268,13 @@ class FileObject():
     @cached_property
     def filesize(self):
         "Filesize in bytes"
-        if self.exists:
-            return self.site.storage.size(self.path)
-        else:
-            # hack for Azure storage
-            # There are no directories in Azure, so they have no size
-            # Return 0 because None causes an exception
-            if hasattr(self.site.storage, 'storage_type'):
-                if self.site.storage.storage_type == 'azure':
-                    return 0
-            return None
+        return self.site.storage.size(self.path) if self.exists else None
 
     @cached_property
     def date(self):
         "Modified time (from site.storage) as float (mktime)"
         if self.exists:
             return time.mktime(get_modified_time(self.site.storage, self.path).timetuple())
-        else:
-            # hack for Azure storage
-            # There are no directories in Azure, get the 'system' file
-            # from 'directory' and return that date
-            if hasattr(self.site.storage, 'storage_type'):
-                if self.site.storage.storage_type == 'azure':
-                    dir_file = self.path+'/dir.azr'
-                    return time.mktime(get_modified_time(self.site.storage, dir_file).timetuple())
         return None
 
     @property
