@@ -1,5 +1,7 @@
+from __future__ import unicode_literals
 import re
 
+import six
 from django.utils.encoding import force_str
 from django.utils.module_loading import import_string
 
@@ -84,11 +86,11 @@ class OptionsNamer(VersionNamer):
             if v is True:
                 opts.append(k)
                 continue
-            if not isinstance(v, str):
+            if not isinstance(v, six.string_types):
                 try:
-                    v = 'x'.join([str(v) for item in v])
+                    v = 'x'.join([six.text_type(v) for item in v])
                 except TypeError:
-                    v = str(v)
+                    v = six.text_type(v)
             opts.append('%s-%s' % (k, v))
 
         return opts
