@@ -3,10 +3,10 @@
 import os
 import sys
 import shutil
-from io import StringIO
 
 from django.conf import settings
 from django.core.management import call_command
+from six import StringIO
 
 from filebrowser.settings import DIRECTORY
 from tests.base import FilebrowserTestCase as TestCase
