@@ -4,7 +4,7 @@ import shutil
 
 from django.conf import settings
 from django.template import Context, Template, TemplateSyntaxError
-from unittest.mock import patch
+from mock import patch
 
 from tests.base import FilebrowserTestCase as TestCase
 from filebrowser.settings import STRICT_PIL
